#include <time.h>

void print(const char *str)
{
    ssize_t len = (ssize_t)strlen(str);
    ssize_t res;

    while (0 < len) {
        res = write(1, str, len);
        if (res < 0) {
            if (errno == EINTR)
                continue;
            perror(PREFIX "print write()");
            exit(1);
        }
        str += res;
        len -= res;
    }
}

int pyopen(const char *filename, int flags)
{
    int fd;
    while (1) {
        errno = 0;
        fd = open(filename, flags);
        if (fd >= 0)
            break;
        if (errno == EINTR) {
#ifdef CHAR
            write(1, CHAR, strlen(CHAR));
#endif
            continue;
        }
        perror(PREFIX "open");
        exit(1);
    }
    return fd;
}

void pysleep(long nsec)
{
    int res;
    struct timespec ts, rem, mon;
    double now, deadline, dt;

    res = clock_gettime(CLOCK_MONOTONIC, &mon);
    assert(res == 0);
    deadline = (double)mon.tv_sec + ((double)mon.tv_nsec + nsec) * 1e-9;

    ts.tv_sec = nsec / (1000 * 1000 * 1000);
    ts.tv_nsec = nsec % (1000 * 1000 * 1000);

    while (1) {
        rem = ts;

        res = nanosleep(&ts, &rem);
        if (!res)
            break;
        if (errno != EINTR) {
            perror(PREFIX "pysleep");
            exit(1);
        }

        res = clock_gettime(CLOCK_MONOTONIC, &mon);
        assert(res == 0);

        now = (double)mon.tv_sec + (double)mon.tv_nsec * 1e-9;
        dt = deadline - now;
        if (dt <= 0)
            break;
        nsec = (long)(dt * 1e9);
        ts.tv_sec = nsec / (1000 * 1000 * 1000);
        ts.tv_nsec = nsec % (1000 * 1000 * 1000);
    }
}

