#include <assert.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <string.h>
#include <unistd.h>

/* #define RANDOM 1 */

#define PREFIX "parent: "
/* #define CHAR "#" */
#include "eintr.h"

volatile int stop = 0;
char filename[100];

#define LOOPS 1000

void sighandler(int signum)
{
/*    write(1, "-", 1); */
}

void sigint(int signum)
{
    stop = 1;

    signal(SIGINT, SIG_IGN);
    signal(SIGTERM, SIG_IGN);
}

void setup(void)
{
    struct itimerval timer;

    signal(SIGALRM, sighandler);
    siginterrupt(SIGALRM, 1);

    signal(SIGINT, sigint);
    signal(SIGTERM, sigint);

    timer.it_value.tv_sec = 0;
    timer.it_value.tv_usec = 10 * 1000;
    timer.it_interval.tv_sec = 0;
    timer.it_interval.tv_usec = 10 * 1000;
    setitimer(ITIMER_REAL, &timer, NULL);
}

void tear_down(void)
{
    struct itimerval timer;

    timer.it_value.tv_sec = 0;
    timer.it_value.tv_usec = 0;
    timer.it_interval.tv_sec = 0;
    timer.it_interval.tv_usec = 0;
    setitimer(ITIMER_REAL, &timer, NULL);
}

pid_t spawn_child(void)
{
    pid_t pid;
    int res;
    char *argv[] = {"./eintr_child", filename, NULL};

    pid = fork();
    if (pid < 0) {
        perror("parent: fork");
        exit(1);
    }

    if (pid) {
        /* parent */
        return pid;
    }

    /* child */
    res = execv(argv[0], argv);

    perror("parent: fork");
    exit(1);
}

void test(int loop)
{
    int res, fd;
    pid_t pid;
    int status;
    long nsec;
    struct stat st;

    snprintf(filename, sizeof(filename), "fifo_%i_%i", getpid(), loop);

    unlink(filename);

    res = mkfifo(filename, 0666);
    if (res != 0) {
        perror("parent: mkfifo");
        exit(1);
    }

    pid = spawn_child();

    fd = pyopen(filename, O_WRONLY);

    res = close(fd);
    if (res < 0) {
        perror("parent: close");
        exit(1);
    }

    res = fstat(fd, &st);
    if (!res)
        print(PREFIX "close() interrupted or failed <========================\n");

    while (1) {
        res = waitpid(pid, &status, 0);
        if (res >= 0)
            break;
        if (errno == EINTR)
            continue;
        perror(PREFIX "waitpid");
        exit(1);
    }

    if (status != 0) {
        print("child failed\n");
        exit(1);
    }

    unlink(filename);
}

int main()
{
    int i;

    setup();

    printf("CTRL+c to stop\n"); fflush(stdout);
    for (i=1; !stop; i++) {
        printf("parent: ctest %i\n", i); fflush(stdout);
        test(i);
    }
    printf("interrupted, exit\n"); fflush(stdout);

    tear_down();

    return 0;
}
